local mainTexture = "Interface\\AddOns\\HavenLoots\\white.tga";
local mainFont = "Interface\\AddOns\\HavenLoots\\Monoglyceride.ttf";
RegisterAddonMessagePrefix("HavenLootsNew");
RegisterAddonMessagePrefix("HavenLootsRoll");
RegisterAddonMessagePrefix("HavenLootsRes");
RegisterAddonMessagePrefix("HavenLootsClear");

--##########################################################################
-- class variables
local itemFrame = {};
local rollWindows = {};
local choiceBoxes = {};
local currentRolls = {};
local hasRolled = {};
local links = {};
for i = 1, 20 do
	itemFrame[i] = OhaiyoClasses_BasicFrame.create();
	choiceBoxes[i] = {};
	rollWindows[i] = {};
	currentRolls[i] = {
		item = nil,
		rollVal = {};
	};
	hasRolled[i] = false;
	for x = 1, 40 do
		rollWindows[i][x] = OhaiyoClasses_BasicFrame.create();
	end

	for x = 1, 4 do
		choiceBoxes[i][x] = OhaiyoClasses_BasicFrame.create();
	end
end
local closeButton = OhaiyoClasses_BasicFrame.create();
local cpos = 0;
local requiredLoot = {};
local requiredLootSize = 0;
local colors = {{1, 0, 0, 1}, {1, 1, 0, 1}, {0, 0, 1, 1}, {0, 1, 0, 1}};
local sendType = "RAID";

--##########################################################################
-- display tooltip
local function ItemButtonUpdate(self, elapsed)
	if(IsModifiedClick('COMPAREITEMS') or (GetCVarBool('alwaysCompareItems') and not IsEquippedItem(self.itemLink))) then
		GameTooltip_ShowCompareItem()
	else
		ShoppingTooltip1:Hide()
		ShoppingTooltip2:Hide()
	end

	if(IsModifiedClick('DRESSUP')) then
		ShowInspectCursor()
	else
		ResetCursor()
	end
end

local function ItemButtonEnter(self)
	GameTooltip:SetOwner(self, 'ANCHOR_TOPRIGHT')
	GameTooltip:SetHyperlink(self.itemLink)
	self:SetScript('OnUpdate', ItemButtonUpdate)
end

local function ItemButtonLeave(self)
	GameTooltip:Hide()
	self:SetScript('OnUpdate', nil)
end

--##########################################################################
-- create loot frame
local function HideRollBoxes(i)
	for x = 1, 4 do
		choiceBoxes[i][x]:Hide();
	end
end

local function ShowRollBoxes(i)
	for x = 1, 4 do
		choiceBoxes[i][x]:Show();
	end
end

local function SendLootRoll(self)
	HideRollBoxes(self.itemNumber);
	hasRolled[self.itemNumber] = true;
	SendAddonMessage("HavenLootsRoll", self.itemNumber .." " ..self.rollType, sendType);
end

local function CreateLootFrame(link, i)
	itemFrame[i]:Show();
	itemFrame[i]:GetFrame().itemLink = link;
	itemFrame[i]:SetText(link);
	currentRolls[i].item = link;
	if hasRolled[i] ~= true then
		ShowRollBoxes(i);
	end
end

local function ClearFrames()
	closeButton:Hide();
	for i = 1, 20 do
		itemFrame[i]:Hide();
		for x = 1, 4 do
			choiceBoxes[i][x]:Hide();
		end
		for x = 1, 40 do
			rollWindows[i][x]:Hide();
		end
	end
end

--##########################################################################
-- set up frames
local function CreateChoiceBox(box, parent, text, type, inum)
	box:PositionFrames("BOTTOMLEFT", parent, "TOPLEFT", 100, 30, 0, 0);
	local c = colors[type];
	box:SetTexture(mainTexture, 0, 0, 0, 0.5);
	box:SetBackground(mainTexture, 1, 1, 1, 1);
	box:SetColor(c[1], c[1], c[3], c[4]);
	box:SetFont(mainFont, 15, 1, 1, 1, 1);
	box:SetText(text);
	box:Hide();

	box:GetFrame():EnableMouse(true);
	box:GetFrame().rollType = type;
	box:GetFrame().itemNumber = inum;
	box:GetFrame():RegisterForDrag("LeftButtonDown");
	box:GetFrame():SetScript("OnMouseDown", SendLootRoll);
end

local function SetUpLootFrames()
	for i = 1, 20 do
		local parent = nil;
		if i == 1 then
			parent = closeButton:GetFrame();
		else
			parent = itemFrame[i - 1]:GetFrame();
		end
		itemFrame[i]:PositionFrames("TOPLEFT", parent, "TOPRIGHT", 100, 30, 0, 0);
		itemFrame[i]:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
		itemFrame[i]:SetBackground(mainTexture, 0, 0, 0, 0.7);
		itemFrame[i]:SetFont(mainFont, 10, 1, 1, 1, 1);
		itemFrame[i]:Hide();

		itemFrame[i]:GetFrame():EnableMouse(true);
		itemFrame[i]:GetFrame():SetScript('OnEnter', ItemButtonEnter);
		itemFrame[i]:GetFrame():SetScript('OnLeave', ItemButtonLeave);

		CreateChoiceBox(choiceBoxes[i][4], itemFrame[i]:GetFrame(), "Main Spec", 4, i);
		choiceBoxes[i][4]:SetColor(0, 1, 0, 1);
		CreateChoiceBox(choiceBoxes[i][3], choiceBoxes[i][4]:GetFrame(), "Off Spec", 3, i);
		choiceBoxes[i][3]:SetColor(0, 0, 1, 1);
		CreateChoiceBox(choiceBoxes[i][2], choiceBoxes[i][3]:GetFrame(), "Alt", 2, i);
		choiceBoxes[i][2]:SetColor(1, 1, 0, 1);
		CreateChoiceBox(choiceBoxes[i][1], choiceBoxes[i][2]:GetFrame(), "Sell", 1, i);
		choiceBoxes[i][1]:SetColor(1, 0, 0, 1);
	end
end

local function SetUpCloseButton()
	closeButton:PositionFrames("TOP", "UIParent", "TOPLEFT", 75, 30, 300, -300);
	closeButton:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
	closeButton:SetBackground(mainTexture, 0, 0, 0, 0.7);
	closeButton:SetFont(mainFont, 15, 1, 1, 1, 1);
	closeButton:SetText("Close");
	closeButton:Hide();

	closeButton:GetFrame():EnableMouse(true);
	closeButton:GetFrame():RegisterForDrag("LeftButtonDown");
	closeButton:GetFrame():SetScript("OnMouseDown",ClearFrames);
end

local function SetUpRollWindows()
	for i = 1, 20 do
		for x = 1, 40 do
			local parent = nil;
			if x == 1 then
				parent = itemFrame[i]:GetFrame();
			else
				parent = rollWindows[i][x - 1]:GetFrame();
			end
			rollWindows[i][x]:PositionFrames("TOPLEFT", parent, "BOTTOMLEFT", 100, 20, 0, 0);
			rollWindows[i][x]:SetTexture(mainTexture, 0, 0, 0, 0.7);
			rollWindows[i][x]:SetFont(mainFont, 10, 1, 1, 1, 1);
			rollWindows[i][x]:SetBackground(mainTexture, 0, 0, 0, 0);
			rollWindows[i][x]:Hide();
		end
	end
end

--##########################################################################
-- add to results table
local function DrawResults()
	for i = 1, 20 do
		if currentRolls[i] ~= nil then
			for x = 1, 40 do
				if currentRolls[i].rollVal[x] ~= nil then
					local c = colors[currentRolls[i].rollVal[x].rollType];
					rollWindows[i][x]:SetColor(c[1], c[2], c[3], c[4]);
					rollWindows[i][x]:SetText(currentRolls[i].rollVal[x].name ..": " ..currentRolls[i].rollVal[x].val - (currentRolls[i].rollVal[x].rollType * 100));
					rollWindows[i][x]:Show();
				end
			end
		end
	end
end

local function AddToResults(name, rollType, val, itemNumber)
	for i = 1, table.getn(currentRolls[itemNumber].rollVal) do
		if currentRolls[itemNumber].rollVal[i].val <= val then
			local data = {
				rollType = rollType,
				val = val,
				name = name,
			};
			table.insert(currentRolls[itemNumber].rollVal, i, data);
			DrawResults();
			return;
		end
	end
	local data = {
		rollType = rollType,
		val = val,
		name = name,
	};
	table.insert(currentRolls[itemNumber].rollVal, data);
	DrawResults();
end

--##########################################################################
-- recieve messages
local getRoll = CreateFrame("FRAME");
getRoll:RegisterEvent("CHAT_MSG_ADDON");
local function GetRoll(self, event, prefix, message, channel, sender)
	if prefix == "HavenLootsNew" then
		cpos = cpos + 1;
		closeButton:Show();
		links[cpos] = message;
		for i = 1, cpos do
			CreateLootFrame(links[i], i);
		end
		DrawResults();
	end
	if prefix == "HavenLootsRes" then
		words = {}
		for word in message:gmatch("%w+") do table.insert(words, word) end
		AddToResults(words[1], tonumber(words[3]), tonumber(words[4]), tonumber(words[5]));
		closeButton:Show();
		for i = 1, cpos do
			CreateLootFrame(links[i], i);
		end
	end
	if prefix == "HavenLootsClear" then
		for i = 1, 20 do
			hasRolled[i] = false;
			currentRolls[i] = {
				item = nil,
				rollVal = {};
			};
		end
		cpos = 0;
		ClearFrames();
	end
end
getRoll:SetScript("OnEvent", GetRoll)

local enterWorld = CreateFrame("FRAME");
enterWorld:RegisterEvent("PLAYER_ENTERING_WORLD");
local function EnterWorld(self, event)
	SetUpCloseButton();
	SetUpRollWindows();
	SetUpLootFrames();
end
enterWorld:SetScript("OnEvent", EnterWorld)

--##########################################################################
-- reopen window
SLASH_HAVENLOOTS1 = '/havenloots';
function SlashCmdList.HAVENLOOTS(msg, editbox)
	closeButton:Show();
	for i = 1, cpos do
		CreateLootFrame(links[i], i);
		DrawResults();
	end
end

--##########################################################################
--https://drive.google.com/folderview?id=0B2n44AM7t7ngfk9QQk5KWlAyVzVPalRyQmFqTnBDY2hDNzRteGhwc0ZkNWVXdXU0N2p6T3c&usp=sharing
-- set roll box color